@@ -9,23 +9,19 @@
 **    May you share freely, never taking more than you give.
 **
 *************************************************************************
-** This file contains the sqlite3_get_table() and sqlite3_free_table()
+** This file contains the sqlite_get_table() and sqlite_free_table()
 ** interface routines.  These are just wrappers around the main
-** interface routine of sqlite3_exec().
+** interface routine of sqlite_exec().
 **
 ** These routines are in a separate files so that they will not be linked
 ** if they are not used.
-**
-** $Id: table.c,v 1.39 2009/01/19 20:49:10 drh Exp $
 */
-#include "sqliteInt.h"
 #include <stdlib.h>
 #include <string.h>
-
-#ifndef SQLITE_OMIT_GET_TABLE
+#include "sqliteInt.h"
 
 /*
-** This structure is used to pass data from sqlite3_get_table() through
+** This structure is used to pass data from sqlite_get_table() through
 ** to the callback function is uses to build the result.
 */
 typedef struct TabResult {
@@ -35,7 +31,7 @@ typedef struct TabResult {
   int nAlloc;
   int nRow;
   int nColumn;
-  int nData;
+  long nData;
   int rc;
 } TabResult;
 
@@ -44,7 +40,7 @@ typedef struct TabResult {
 ** is to fill in the TabResult structure appropriately, allocating new
 ** memory as necessary.
 */
-static int sqlite3_get_table_cb(void *pArg, int nCol, char **argv, char **colv){
+static int sqlite_get_table_cb(void *pArg, int nCol, char **argv, char **colv){
   TabResult *p = (TabResult*)pArg;
   int need;
   int i;
@@ -61,8 +57,11 @@ static int sqlite3_get_table_cb(void *pArg, int nCol, char **argv, char **colv){
   if( p->nData + need >= p->nAlloc ){
     char **azNew;
     p->nAlloc = p->nAlloc*2 + need + 1;
-    azNew = sqlite3_realloc( p->azResult, sizeof(char*)*p->nAlloc );
-    if( azNew==0 ) goto malloc_failed;
+    azNew = realloc( p->azResult, sizeof(char*)*p->nAlloc );
+    if( azNew==0 ){
+      p->rc = SQLITE_NOMEM;
+      return 1;
+    }
     p->azResult = azNew;
   }
 
@@ -72,15 +71,22 @@ static int sqlite3_get_table_cb(void *pArg, int nCol, char **argv, char **colv){
   if( p->nRow==0 ){
     p->nColumn = nCol;
     for(i=0; i<nCol; i++){
-      z = sqlite3_mprintf("%s", colv[i]);
-      if( z==0 ) goto malloc_failed;
+      if( colv[i]==0 ){
+        z = 0;
+      }else{
+        z = malloc( strlen(colv[i])+1 );
+        if( z==0 ){
+          p->rc = SQLITE_NOMEM;
+          return 1;
+        }
+        strcpy(z, colv[i]);
+      }
       p->azResult[p->nData++] = z;
     }
   }else if( p->nColumn!=nCol ){
-    sqlite3_free(p->zErrMsg);
-    p->zErrMsg = sqlite3_mprintf(
-       "sqlite3_get_table() called with two or more incompatible queries"
-    );
+    sqliteSetString(&p->zErrMsg,
+       "sqlite_get_table() called with two or more incompatible queries",
+       (char*)0);
     p->rc = SQLITE_ERROR;
     return 1;
   }
@@ -92,20 +98,18 @@ static int sqlite3_get_table_cb(void *pArg, int nCol, char **argv, char **colv){
       if( argv[i]==0 ){
         z = 0;
       }else{
-        int n = sqlite3Strlen30(argv[i])+1;
-        z = sqlite3_malloc( n );
-        if( z==0 ) goto malloc_failed;
-        memcpy(z, argv[i], n);
+        z = malloc( strlen(argv[i])+1 );
+        if( z==0 ){
+          p->rc = SQLITE_NOMEM;
+          return 1;
+        }
+        strcpy(z, argv[i]);
       }
       p->azResult[p->nData++] = z;
     }
     p->nRow++;
   }
   return 0;
-
-malloc_failed:
-  p->rc = SQLITE_NOMEM;
-  return 1;
 }
 
 /*
@@ -115,11 +119,11 @@ malloc_failed:
 **
 ** The result that is written to ***pazResult is held in memory obtained
 ** from malloc().  But the caller cannot free this memory directly.  
-** Instead, the entire table should be passed to sqlite3_free_table() when
+** Instead, the entire table should be passed to sqlite_free_table() when
 ** the calling procedure is finished using it.
 */
-int sqlite3_get_table(
-  sqlite3 *db,                /* The database on which the SQL executes */
+int sqlite_get_table(
+  sqlite *db,                 /* The database on which the SQL executes */
   const char *zSql,           /* The SQL to be executed */
   char ***pazResult,          /* Write the result table here */
   int *pnRow,                 /* Write the number of rows in the result here */
@@ -128,11 +132,10 @@ int sqlite3_get_table(
 ){
   int rc;
   TabResult res;
-
+  if( pazResult==0 ){ return SQLITE_ERROR; }
   *pazResult = 0;
   if( pnColumn ) *pnColumn = 0;
   if( pnRow ) *pnRow = 0;
-  if( pzErrMsg ) *pzErrMsg = 0;
   res.zErrMsg = 0;
   res.nResult = 0;
   res.nRow = 0;
@@ -140,38 +143,38 @@ int sqlite3_get_table(
   res.nData = 1;
   res.nAlloc = 20;
   res.rc = SQLITE_OK;
-  res.azResult = sqlite3_malloc(sizeof(char*)*res.nAlloc );
+  res.azResult = malloc( sizeof(char*)*res.nAlloc );
   if( res.azResult==0 ){
-     db->errCode = SQLITE_NOMEM;
-     return SQLITE_NOMEM;
+    return SQLITE_NOMEM;
   }
   res.azResult[0] = 0;
-  rc = sqlite3_exec(db, zSql, sqlite3_get_table_cb, &res, pzErrMsg);
-  assert( sizeof(res.azResult[0])>= sizeof(res.nData) );
-  res.azResult[0] = SQLITE_INT_TO_PTR(res.nData);
-  if( (rc&0xff)==SQLITE_ABORT ){
-    sqlite3_free_table(&res.azResult[1]);
+  rc = sqlite_exec(db, zSql, sqlite_get_table_cb, &res, pzErrMsg);
+  if( res.azResult ){
+    res.azResult[0] = (char*)res.nData;
+  }
+  if( rc==SQLITE_ABORT ){
+    sqlite_free_table(&res.azResult[1]);
     if( res.zErrMsg ){
       if( pzErrMsg ){
-        sqlite3_free(*pzErrMsg);
-        *pzErrMsg = sqlite3_mprintf("%s",res.zErrMsg);
+        free(*pzErrMsg);
+        *pzErrMsg = res.zErrMsg;
+        sqliteStrRealloc(pzErrMsg);
+      }else{
+        sqliteFree(res.zErrMsg);
       }
-      sqlite3_free(res.zErrMsg);
     }
-    db->errCode = res.rc;  /* Assume 32-bit assignment is atomic */
     return res.rc;
   }
-  sqlite3_free(res.zErrMsg);
+  sqliteFree(res.zErrMsg);
   if( rc!=SQLITE_OK ){
-    sqlite3_free_table(&res.azResult[1]);
+    sqlite_free_table(&res.azResult[1]);
     return rc;
   }
   if( res.nAlloc>res.nData ){
     char **azNew;
-    azNew = sqlite3_realloc( res.azResult, sizeof(char*)*(res.nData+1) );
+    azNew = realloc( res.azResult, sizeof(char*)*(res.nData+1) );
     if( azNew==0 ){
-      sqlite3_free_table(&res.azResult[1]);
-      db->errCode = SQLITE_NOMEM;
+      sqlite_free_table(&res.azResult[1]);
       return SQLITE_NOMEM;
     }
     res.nAlloc = res.nData+1;
@@ -184,19 +187,17 @@ int sqlite3_get_table(
 }
 
 /*
-** This routine frees the space the sqlite3_get_table() malloced.
+** This routine frees the space the sqlite_get_table() malloced.
 */
-void sqlite3_free_table(
-  char **azResult            /* Result returned from from sqlite3_get_table() */
+void sqlite_free_table(
+  char **azResult             /* Result returned from from sqlite_get_table() */
 ){
   if( azResult ){
     int i, n;
     azResult--;
-    assert( azResult!=0 );
-    n = SQLITE_PTR_TO_INT(azResult[0]);
-    for(i=1; i<n; i++){ if( azResult[i] ) sqlite3_free(azResult[i]); }
-    sqlite3_free(azResult);
+    if( azResult==0 ) return;
+    n = (int)(long)azResult[0];
+    for(i=1; i<n; i++){ if( azResult[i] ) free(azResult[i]); }
+    free(azResult);
   }
 }
-
-#endif /* SQLITE_OMIT_GET_TABLE */
